#!/usr/local/bin/perl

print <<"_EOT_";
-----%-----%-----%-----%-----%-----%-----%-----%-----%-----%-----
       abs2rel.pl
           Change Absolute URL to Relative URL
       by Yoshioka Tsuneo(tsuneo-y\@is.aist-nara.ac.jp,QWF00133\@nifty.ne.jp)
       please use as Public Domain Software.
       Copy,Edit,Re-distribute FREE
       Welcome any e-mail.
-----%-----%-----%-----%-----%-----%-----%-----%-----%-----%-----
_EOT_

local($rootdir) = ".";
local($htmlext) = '\.s?html?$';
local($testmode) = 0;
local($backup) = 0;

sub usage
{
    print "usage: abs2rel.pl [--rootdir=<dir>] [--htmlext=<regexp>] [--test] [--backup]\n";
}
while($_ = $ARGV[0]){
    if(/^--rootdir=/){
	$rootdir = $';
    }elsif(/^--htmlext=/){
	$htmlext = $';
    }elsif(/^--test/){
	$testmode = 1;
    }elsif(/^--backup/){
	$backup = 1;
    }else{
	&usage;
    }
    shift;
}
chdir($rootdir) || die "can't chdir to [$rootdir]:$!";
&abs2rel_dir("http:/","http");

sub abs2rel_url
{
    local($currenturl,$url) = @_;
#    local($urlhead,$urlbody);
    local($currenturl0,$url0) = ($currenturl,$url);
    local($host_root_dir,$host_dir);

    $currenturl =~ s|[^/]*$||;

    if($currenturl =~ m|^(http://[^/]+)(.*)|){
#	$urlhead = $1;
#	$urlbody = $2;
	$host_root_dir = ("../" x (($urlbody =~ tr|/|/|)-1));
	$host_dir = "../" . $host_root_dir;
    }
    if($url =~ s|^/||){
	$url = $host_root_dir . $url;
    }elsif($url =~ s|^http://||){
	$url = $host_dir . $url;
    }
    print "(CURRENTURL,URL,NEWURL)=($currenturl0,$url0,$url)\n";
    return $url;
}
sub abs2rel_str
{
    local($currenturl,*fromfile) = @_;
    if($fromfile =~ /[\0\007\177]/){
	print "this file may be binary\n";
	return;
    }
    # print "FROMFILE=($fromfile)\n";
#    $fromfile =~ s|<[^<>]*["=\s'](http://[^<>"'=\s?]+)["=\s'>]|&abs2rel_url($currenturl,$1)|eg;
    $fromfile =~ s#(<[^<>]*(src|href)\s*=\s*["']?)([^<>"'=\s?]+)(["=\s'>])#$1 . &abs2rel_url($currenturl,$3) . $4#ieg;
}
sub abs2rel_file
{
    local($currenturl,$file) = @_;
    local(*FH_R,*FH_W);
    local($fromfile);

    if((-s $file) > 300000){
	print "INFO: $file is too large. skipped \n";
	return;
    }
    link $file,"$file.bak" || die "can't link [$file] to [$file.bak]:$!";
    open(FH_R,"$file.bak") || die "ERROR: can't open read file[$file.bak]:$!";
    $fromfile = join('',<FH_R>);
    close(FH_R);
    &abs2rel_str($currenturl,*fromfile);
    open(FH_W,">$file.new") || die "ERROR: can't open write file[$file.new]:$!";
    print FH_W $fromfile;
    close(FH_W);
    print "cwd=[",`pwd`,"]\n";
    print "=========$file.new created=====\n";
#    print "FROMFILE=($fromfile)\n";
    if(! $testmode){
	unlink $file;
	link "$file.new",$file || die "can't link [$file.new] to [$file]:$!";
	unlink "$file.new";
    }
    if(! $backup){
	unlink "$file.bak" || die "can't unlink [$file.bak]:$!";
    }
}
sub abs2rel_dir
{
    local($currenturl,$dir) = @_;
    local($file,$afile);
    local(@files);
    local($url);
    local(*DH);

 #   chdir($dir) || die "can't chdir [$dir]:$!";
    opendir(DH,$dir) || die "can't opendir [$dir]:$!";
    @files = grep {$_ ne "." && $_ ne ".."} readdir(DH);
    closedir(DH);
 #  local(@files) = <$dir/*>;
    print "files=[",join(',',@files),"]\n";
    foreach $file(@files){
	$url = $currenturl . "/" . $file;
	$afile = $dir . "/" . $file;
	print "(URL,FILE)=($url,$afile)\n";
	if(-d $afile){
	    &abs2rel_dir($url,$afile);
	}elsif($url =~ /$htmlext/i){
	    &abs2rel_file($url,$afile);
	}
    }
 #   chdir("..");
}
